/******************************************************************
 *****                                                        *****
 *****  Name: cs8900.c                                        *****
 *****  Ver.: 1.0                                             *****
 *****  Date: 07/05/2001                                      *****
 *****  Auth: Andreas Dannenberg                              *****
 *****        HTWK Leipzig                                    *****
 *****        university of applied sciences                  *****
 *****        Germany                                         *****
 *****  Func: ethernet packet-driver for use with LAN-        *****
 *****        controller CS8900 from Crystal/Cirrus Logic     *****
 *****                                                        *****
 *****  Keil: Module modified for use with Philips            *****
 *****        LPC2378 EMAC Ethernet controller                *****
 *****                                                        *****
 ******************************************************************/

#include "EMAC.h"
#include "tcpip.h"
#include "LPC17xx.h"

static unsigned short *rptr;
static unsigned short *tptr;

// Keil: function added to write PHY
void write_PHY (int PhyReg, int Value)
{
  unsigned int tout;

  LPC_EMAC->MADR = DM9161A_DEF_ADR | PhyReg;
  LPC_EMAC->MWTD = Value;

  /* Wait utill operation completed */
  tout = 0;
  for (tout = 0; tout < MII_WR_TOUT; tout++) {
    if ((LPC_EMAC->MIND & MIND_BUSY) == 0) {
      break;
    }
  }
}


// Keil: function added to read PHY
unsigned short read_PHY (unsigned char PhyReg) 
{
  unsigned int tout;

  LPC_EMAC->MADR = DM9161A_DEF_ADR | PhyReg;
  LPC_EMAC->MCMD = MCMD_READ;

  /* Wait until operation completed */
  tout = 0;
  for (tout = 0; tout < MII_RD_TOUT; tout++) {
    if ((LPC_EMAC->MIND & MIND_BUSY) == 0) {
      break;
    }
  }
  LPC_EMAC->MCMD = 0;
  return (LPC_EMAC->MRDD);
}


// Keil: function added to initialize Rx Descriptors
void rx_descr_init (void)
{
  unsigned int i;

  for (i = 0; i < NUM_RX_FRAG; i++) {
    RX_DESC_PACKET(i)  = RX_BUF(i);
    RX_DESC_CTRL(i)    = RCTRL_INT | (ETH_FRAG_SIZE-1);
    RX_STAT_INFO(i)    = 0;
    RX_STAT_HASHCRC(i) = 0;
  }

  /* Set EMAC Receive Descriptor Registers. */
  LPC_EMAC->RxDescriptor    = RX_DESC_BASE;
  LPC_EMAC->RxStatus        = RX_STAT_BASE;
  LPC_EMAC->RxDescriptorNumber = NUM_RX_FRAG-1;

  /* Rx Descriptors Point to 0 */
  LPC_EMAC->RxConsumeIndex  = 0;
}


// Keil: function added to initialize Tx Descriptors
void tx_descr_init (void) {
  unsigned int i;

  for (i = 0; i < NUM_TX_FRAG; i++) {
    TX_DESC_PACKET(i) = TX_BUF(i);
    TX_DESC_CTRL(i)   = 0;
    TX_STAT_INFO(i)   = 0;
  }

  /* Set EMAC Transmit Descriptor Registers. */
  LPC_EMAC->TxDescriptor    = TX_DESC_BASE;
  LPC_EMAC->TxStatus        = TX_STAT_BASE;
  LPC_EMAC->TxDescriptorNumber = NUM_TX_FRAG-1;

  /* Tx Descriptors Point to 0 */
  LPC_EMAC->TxProduceIndex  = 0;
}


// configure port-pins for use with LAN-controller,
// reset it and send the configuration-sequence

void Init_EMAC(void)
{
unsigned int regv,tout;
unsigned short i,PHY_ADDR;

	/* Power Up the EMAC controller. */
	LPC_SC->PCONP |= 0x40000000;
	
	/* Enable P1 and P2(LPC1758) Ethernet Pins. */
    LPC_PINCON->PINSEL2 |= 0x50150105;					// selects P1[0,1,4,8,9,10,14,15]
	
//	MIDO_INIT();										// ---LPC175X
	LPC_PINCON->PINSEL3 |= 0x00000005;				// selects P1[17:16]  ---LPC176X

	/* Reset all EMAC internal modules. */
	LPC_EMAC->MAC1 = MAC1_RES_TX | MAC1_RES_MCS_TX | MAC1_RES_RX | MAC1_RES_MCS_RX | MAC1_SIM_RES | MAC1_SOFT_RES;
	LPC_EMAC->Command = CR_REG_RES | CR_TX_RES | CR_RX_RES;
  
	/* A short delay after reset. */
	for (tout = 100; tout; tout--);

	/* Initialize MAC control registers. */
	LPC_EMAC->MAC1 = MAC1_PASS_ALL;					//Pass All Rx Frame.
	LPC_EMAC->MAC2 = MAC2_CRC_EN | MAC2_PAD_EN;	 	//CRC and PAD enabled.  --- 60 Bytes.

	LPC_EMAC->MAXF = ETH_MAX_FLEN;
	LPC_EMAC->CLRT = CLRT_DEF;
	LPC_EMAC->IPGR = IPGR_DEF;

	/* MII clock MAX=2.5M ! Host clock divided by 52 of AHB(=100M), no suppress preamble, no scan increment */
	LPC_EMAC->MCFG |= 0x0018;
	
	/* Enable Reduced MII interface. */
	LPC_EMAC->Command = CR_RMII | CR_PASS_RUNT_FRM;
	
	/* RMII setting, PHY support: [8]=0 ->10 Mbps mode, =1 -> 100 Mbps mode */
	//LPC_EMAC->SUPP = 0x0900;	


//	printf("PHY_ADDR=%d\r\n",PHY_ADDR);

	/* Put the DM9161A in reset mode. */
	write_PHY( DM9161_BMCR, DM9161_RESET);

	/* Wait for hardware reset to end. */
	for (tout = 0; tout < 0x400000; tout++) 
		{
    	regv = read_PHY( DM9161_BMCR);
    	if (!(regv & 0x8000)) 
			{
      		/* Reset complete */
      		break;
    		}
		}

//	printf("DM9161_BMCR=%d\r\n",regv);

    /* Use autonegotiation about the link speed. */
//  write_PHY( DM9161_BMCR, (DM9161_AUTONEG|DM9161_RESTART_AUTONEG) );
    
	/* Connect at 10MBit full-duplex */
	write_PHY (DM9161_BMCR, EMAC_PHY_FULLD_10M);

	/* Wait to complete Auto_Negotiation. */
    for (tout = 0; tout < 0x4000; tout++) 
		{
      	regv = read_PHY( DM9161_BMSR);
      	if (regv & 0x0020) 
			{
        	/* Autonegotiation Complete. */
 //       	printf( "tout=%d\r\n",tout);
			break;
			
			}
		}

//	printf( "Auto_Negotiation=%d\r\n",regv);

	/* Check the link status. */
	for (tout = 0; tout < 0x40000; tout++) 
		{
    	regv = read_PHY( DM9161_DSCSR);
    	if( (regv & 0x8000)||(regv & 0x4000)||(regv & 0x2000)||(regv & 0x1000) ) 
			{
      		/* Link is on. */
      		break;
			}
		}

//	printf( "DM9161_DSCSR=%d\r\n",regv);

	/* Configure Full/Half Duplex mode and 100MBit/10MBit mode. ---EMAC */
//	if(regv & 0x8000) 
		{
		/* Full duplex mode---100M. */
//		LPC_EMAC->MAC2    |= MAC2_FULL_DUP;
//		LPC_EMAC->Command |= CR_FULL_DUP;
//		LPC_EMAC->SUPP     = SUPP_SPEED;
//		LPC_EMAC->IPGT     = IPGT_FULL_DUP;
		}
//	else if(regv & 0x4000)
		{
		/* Half duplex mode---100M. */
//		LPC_EMAC->SUPP     = SUPP_SPEED;
//		LPC_EMAC->IPGT 	   = IPGT_HALF_DUP;
		}
//	else if(regv & 0x2000)
		{
    	/* Full duplex mode---10M. */
		LPC_EMAC->MAC2    |= MAC2_FULL_DUP;
		LPC_EMAC->Command |= CR_FULL_DUP;
		LPC_EMAC->SUPP     = 0;
		LPC_EMAC->IPGT     = IPGT_FULL_DUP;
  		}
//	else if(regv & 0x1000)
  		{
    	/* Half duplex mode---10M. */
//		LPC_EMAC->SUPP     = 0;
//		LPC_EMAC->IPGT 	   = IPGT_HALF_DUP;
  		}

//	printf( "Go to this place !!!------1\r\n");

  /* Set the Ethernet MAC Address registers */
  LPC_EMAC->SA0 = (MYMAC_1 << 8) | MYMAC_2;
  LPC_EMAC->SA1 = (MYMAC_3 << 8) | MYMAC_4;
  LPC_EMAC->SA2 = (MYMAC_5 << 8) | MYMAC_6;


	/* Initialize Tx and Rx DMA Descriptors */
	tx_descr_init();
	rx_descr_init();

//	printf( "Go to this place !!!------2\r\n");

	/* Receive Broadcast and Perfect Match Packets */
	LPC_EMAC->RxFilterCtrl = RFC_BCAST_EN | RFC_PERFECT_EN;					 

	/* Enable EMAC interrupts. */
	//LPC_EMAC->IntEnable = INT_RX_DONE | INT_TX_DONE;

//	printf( "Go to this place !!!------3\r\n");

	/* Reset all interrupts */
	//LPC_EMAC->IntClear  = 0xFFFF;

	/* Enable receive and transmit mode of MAC Ethernet core */
	LPC_EMAC->Command  |= CR_RX_EN | CR_TX_EN;
	LPC_EMAC->MAC1     |= MAC1_REC_EN;

//	printf( "Go to this place !!!------4\r\n");
}


// reads a word in little-endian byte order from RX_BUFFER

unsigned short ReadFrame_EMAC(void)
{
  return (*rptr++);
}

// reads a word in big-endian byte order from RX_FRAME_PORT
// (useful to avoid permanent byte-swapping while reading
// TCP/IP-data)

unsigned short ReadFrameBE_EMAC(void)
{
  unsigned short ReturnValue;

  ReturnValue = SwapBytes (*rptr++);
  return (ReturnValue);
}


// copies bytes from frame port to MCU-memory
// NOTES: * an odd number of byte may only be transfered
//          if the frame is read to the end!
//        * MCU-memory MUST start at word-boundary

void CopyFromFrame_EMAC(void *Dest, unsigned short Size)
{
  unsigned short * piDest;                       // Keil: Pointer added to correct expression

  piDest = Dest;                                 // Keil: Line added
  while (Size > 1) {
    *piDest++ = ReadFrame_EMAC();
    Size -= 2;
  }
  
  if (Size) {                                         // check for leftover byte...
    *(unsigned char *)piDest = (char)ReadFrame_EMAC();// the LAN-Controller will return 0
  }                                                   // for the highbyte
}

// does a dummy read on frame-I/O-port
// NOTE: only an even number of bytes is read!

void DummyReadFrame_EMAC(unsigned short Size)    // discards an EVEN number of bytes
{                                                // from RX-fifo
  while (Size > 1) {
    ReadFrame_EMAC();
    Size -= 2;
  }
}

// Reads the length of the received ethernet frame and checks if the 
// destination address is a broadcast message or not
// returns the frame length
unsigned short StartReadFrame(void) {
  unsigned short RxLen;
  unsigned int idx;

  idx = LPC_EMAC->RxConsumeIndex;
  RxLen = (RX_STAT_INFO(idx) & RINFO_SIZE) - 3;
  rptr = (unsigned short *)RX_DESC_PACKET(idx);
  return(RxLen);
}

void EndReadFrame(void) {
  unsigned int idx;

  /* DMA free packet. */
  idx = LPC_EMAC->RxConsumeIndex;
  if (++idx == NUM_RX_FRAG) idx = 0;
  LPC_EMAC->RxConsumeIndex = idx;
}

unsigned int CheckFrameReceived(void) {             // Packet received ?

  if (LPC_EMAC->RxProduceIndex != LPC_EMAC->RxConsumeIndex) // more packets received ?		
    return(1);
  else 
    return(0);
}

// requests space in EMAC memory for storing an outgoing frame

void RequestSend(unsigned short FrameSize)
{
  unsigned int idx;

  idx  = LPC_EMAC->TxProduceIndex;
  tptr = (unsigned short *)TX_DESC_PACKET(idx);
  TX_DESC_CTRL(idx) = FrameSize | TCTRL_LAST;
}

// check if ethernet controller is ready to accept the
// frame we want to send

unsigned int Rdy4Tx(void)
{
  return (1);   // the ethernet controller transmits much faster
}               // than the CPU can load its buffers


// writes a word in little-endian byte order to TX_BUFFER
void WriteFrame_EMAC(unsigned short Data)
{
  *tptr++ = Data;
}

// copies bytes from MCU-memory to frame port
// NOTES: * an odd number of byte may only be transfered
//          if the frame is written to the end!
//        * MCU-memory MUST start at word-boundary

void CopyToFrame_EMAC(void *Source, unsigned int Size)
{
  unsigned short * piSource;
  unsigned int idx;

  piSource = Source;
  Size = (Size + 1) & 0xFFFE;    // round Size up to next even number
  while (Size > 0) {
    WriteFrame_EMAC(*piSource++);
    Size -= 2;
  }

  idx = LPC_EMAC->TxProduceIndex;
  if (++idx == NUM_TX_FRAG) idx = 0;
  LPC_EMAC->TxProduceIndex = idx;
}

